@@ -7,9 +7,10 @@ import uuid
 
 from future.standard_library import install_aliases
 
-from pyngrok import process
+from pyngrok import process, conf
+from pyngrok.conf import PyngrokConfig
 from pyngrok.exception import PyngrokNgrokHTTPError, PyngrokNgrokURLError, PyngrokSecurityError
-from pyngrok.installer import get_ngrok_bin, install_ngrok, install_default_config
+from pyngrok.installer import install_ngrok, install_default_config
 
 install_aliases()
 
@@ -26,27 +27,27 @@ except ImportError:  # pragma: no cover
 
 __author__ = "Alex Laird"
 __copyright__ = "Copyright 2020, Alex Laird"
-__version__ = "2.1.6"
+__version__ = "4.0.1"
 
 logger = logging.getLogger(__name__)
 
-BIN_DIR = os.path.normpath(os.path.join(os.path.abspath(os.path.dirname(__file__)), "bin"))
-DEFAULT_NGROK_PATH = os.path.join(BIN_DIR, get_ngrok_bin())
-DEFAULT_CONFIG_PATH = None
-
-_DEFAULT_NGROK_CONFIG_PATH = os.path.join(os.path.expanduser("~"), ".ngrok2", "ngrok.yml")
-
 
 class NgrokTunnel:
     """
     An object containing information about an `ngrok` tunnel.
 
-    :var string name: The name of the tunnel.
-    :var string proto: A valid `tunnel protocol <https://ngrok.com/docs#tunnel-definitions>`_.
-    :var string uri: The tunnel URI, a relative path that can be used to make requests to the `ngrok` web interface.
-    :var string public_url: The public `ngrok` URL.
-    :var dict config: The config for the tunnel.
-    :var dict metrics: Metrics for `the tunnel <https://ngrok.com/docs#list-tunnels>`_.
+    :var name: The name of the tunnel.
+    :vartype name: str
+    :var proto: A valid `tunnel protocol <https://ngrok.com/docs#tunnel-definitions>`_.
+    :vartype proto: str
+    :var uri: The tunnel URI, a relative path that can be used to make requests to the `ngrok` web interface.
+    :vartype uri: str
+    :var public_url: The public `ngrok` URL.
+    :vartype public_url: str
+    :var config: The config for the tunnel.
+    :vartype config: dict
+    :var metrics: Metrics for `the tunnel <https://ngrok.com/docs#list-tunnels>`_.
+    :vartype metrics: dict
     """
 
     def __init__(self, data=None):
@@ -75,16 +76,16 @@ def ensure_ngrok_installed(ngrok_path):
     the current system if not.
 
     :param ngrok_path: The path to the `ngrok` binary.
-    :type ngrok_path: string
+    :type ngrok_path: str
     """
     if not os.path.exists(ngrok_path):
         install_ngrok(ngrok_path)
 
-    if not os.path.exists(_DEFAULT_NGROK_CONFIG_PATH):
-        install_default_config(_DEFAULT_NGROK_CONFIG_PATH)
+    if not os.path.exists(conf.DEFAULT_NGROK_CONFIG_PATH):
+        install_default_config(conf.DEFAULT_NGROK_CONFIG_PATH)
 
 
-def set_auth_token(token, ngrok_path=None, config_path=None):
+def set_auth_token(token, pyngrok_config=None):
     """
     Set the `ngrok` auth token in the config file, enabling authenticated features (for instance,
     more concurrent tunnels, custom subdomains, etc.).
@@ -93,21 +94,19 @@ def set_auth_token(token, ngrok_path=None, config_path=None):
     and install `ngrok`.
 
     :param token: The auth token to set.
-    :type token: string
-    :param ngrok_path: A `ngrok` binary override (instead of using `pyngrok`'s).
-    :type ngrok_path: string, optional
-    :param config_path: A config path override.
-    :type config_path: string, optional
+    :type token: str
+    :param pyngrok_config: The Pyngrok configuration to use when with `ngrok`.
+    :type pyngrok_config: PyngrokConfig, optional
     """
-    ngrok_path = ngrok_path if ngrok_path else DEFAULT_NGROK_PATH
-    config_path = config_path if config_path else DEFAULT_CONFIG_PATH
+    if pyngrok_config is None:
+        pyngrok_config = PyngrokConfig()
 
-    ensure_ngrok_installed(ngrok_path)
+    ensure_ngrok_installed(pyngrok_config.ngrok_path)
 
-    process.set_auth_token(ngrok_path, token, config_path)
+    process.set_auth_token(pyngrok_config, token)
 
 
-def get_ngrok_process(ngrok_path=None, config_path=None, auth_token=None, region=None):
+def get_ngrok_process(pyngrok_config=None):
     """
     Retrieve the current `ngrok` process for the given path.
 
@@ -116,27 +115,20 @@ def get_ngrok_process(ngrok_path=None, config_path=None, auth_token=None, region
 
     If `ngrok` is not running, calling this method will start a process for the given path.
 
-    :param ngrok_path: A `ngrok` binary override (instead of using `pyngrok`'s).
-    :type ngrok_path: string, optional
-    :param config_path: A config path override. Ignored if `ngrok` is already running.
-    :type config_path: string, optional
-    :param auth_token: An authtoken override. Ignored if `ngrok` is already running.
-    :type auth_token: string, optional
-    :param region: A region override. Ignored if `ngrok` is already running.
-    :type region: string, optional
+    :param pyngrok_config: The Pyngrok configuration to use when with `ngrok`.
+    :type pyngrok_config: PyngrokConfig, optional
     :return: The `ngrok` process.
     :rtype: NgrokProcess
     """
-    ngrok_path = ngrok_path if ngrok_path else DEFAULT_NGROK_PATH
-    config_path = config_path if config_path else DEFAULT_CONFIG_PATH
+    if pyngrok_config is None:
+        pyngrok_config = PyngrokConfig()
 
-    ensure_ngrok_installed(ngrok_path)
+    ensure_ngrok_installed(pyngrok_config.ngrok_path)
 
-    return process.get_process(ngrok_path, config_path, auth_token, region)
+    return process.get_process(pyngrok_config)
 
 
-def connect(port=80, proto="http", name=None, options=None, ngrok_path=None, config_path=None, timeout=4,
-            auth_token=None, region=None):
+def connect(port=80, proto="http", name=None, options=None, pyngrok_config=None):
     """
     Establish a new `ngrok` tunnel to the given port and protocol, returning the connected
     public URL that tunnels to the local port.
@@ -149,42 +141,34 @@ def connect(port=80, proto="http", name=None, options=None, ngrok_path=None, con
     :param port: The local port to which to tunnel, defaults to 80.
     :type port: int, optional
     :param proto: The protocol to tunnel, defaults to "http".
-    :type proto: string, optional
+    :type proto: str, optional
     :param name: A friendly name for the tunnel.
-    :type name: string, optional
+    :type name: str, optional
     :param options: Parameters passed to `configuration for the ngrok tunnel <https://ngrok.com/docs#tunnel-definitions>`_.
-    :type options: dict, optional
-    :param ngrok_path: A `ngrok` binary override (instead of using `pyngrok`'s).
-    :type ngrok_path: string, optional
-    :param config_path: A config path override. Ignored if `ngrok` is already running.
-    :type config_path: string, optional
-    :param timeout: The request timeout, in seconds.
-    :type timeout: float, optional
-    :param auth_token: An authtoken override. Ignored if `ngrok` is already running.
-    :type auth_token: string, optional
-    :param region: A region override. Ignored if `ngrok` is already running.
-    :type region: string, optional
+    :type options: dict[str, str], optional
+    :param pyngrok_config: The Pyngrok configuration to use when with `ngrok`.
+    :type pyngrok_config: PyngrokConfig, optional
     :return: The connected public URL.
-    :rtype: string
+    :rtype: str
     """
     if options is None:
         options = {}
-
-    ngrok_path = ngrok_path if ngrok_path else DEFAULT_NGROK_PATH
-    config_path = config_path if config_path else DEFAULT_CONFIG_PATH
+    if pyngrok_config is None:
+        pyngrok_config = PyngrokConfig()
 
     config = {
-        "name": name if name else str(uuid.uuid4()),
+        "name": name if name else "{}-{}-{}".format(proto, port, uuid.uuid4()),
         "addr": str(port),
         "proto": proto
     }
     options.update(config)
 
-    api_url = get_ngrok_process(ngrok_path, config_path, auth_token, region).api_url
+    api_url = get_ngrok_process(pyngrok_config).api_url
 
     logger.debug("Connecting tunnel with options: {}".format(options))
 
-    tunnel = NgrokTunnel(api_request("{}/api/{}".format(api_url, "tunnels"), "POST", data=options, timeout=timeout))
+    tunnel = NgrokTunnel(api_request("{}/api/{}".format(api_url, "tunnels"), method="POST", data=options,
+                                     timeout=pyngrok_config.request_timeout))
 
     if proto == "http" and ("bind_tls" not in options or options["bind_tls"] != False):
         tunnel.public_url = tunnel.public_url.replace("https", "http")
@@ -192,7 +176,7 @@ def connect(port=80, proto="http", name=None, options=None, ngrok_path=None, con
     return tunnel.public_url
 
 
-def disconnect(public_url, ngrok_path=None, config_path=None, timeout=4):
+def disconnect(public_url, pyngrok_config=None):
     """
     Disconnect the `ngrok` tunnel for the given URL.
 
@@ -202,29 +186,27 @@ def disconnect(public_url, ngrok_path=None, config_path=None, timeout=4):
     If `ngrok` is not running, calling this method will start a process for the given path.
 
     :param public_url: The public URL of the tunnel to disconnect.
-    :type public_url: string
-    :param ngrok_path: A `ngrok` binary override (instead of using `pyngrok`'s).
-    :type ngrok_path: string, optional
-    :param config_path: A config path override.
-    :type config_path: string, optional
-    :param timeout: The request timeout, in seconds.
-    :type timeout: float, optional
+    :type public_url: str
+    :param pyngrok_config: The Pyngrok configuration to use when with `ngrok`.
+    :type pyngrok_config: PyngrokConfig, optional
     """
-    ngrok_path = ngrok_path if ngrok_path else DEFAULT_NGROK_PATH
+    if pyngrok_config is None:
+        pyngrok_config = PyngrokConfig()
 
-    api_url = get_ngrok_process(ngrok_path, config_path).api_url
+    api_url = get_ngrok_process(pyngrok_config).api_url
 
-    tunnels = get_tunnels(ngrok_path)
+    tunnels = get_tunnels(pyngrok_config)
     for tunnel in tunnels:
         if tunnel.public_url == public_url:
             logger.debug("Disconnecting tunnel: {}".format(tunnel.public_url))
 
-            api_request("{}{}".format(api_url, tunnel.uri.replace("+", "%20")), "DELETE", timeout=timeout)
+            api_request("{}{}".format(api_url, tunnel.uri.replace("+", "%20")), method="DELETE",
+                        timeout=pyngrok_config.request_timeout)
 
             break
 
 
-def get_tunnels(ngrok_path=None, timeout=4):
+def get_tunnels(pyngrok_config=None):
     """
     Retrieve a list of all active `ngrok` tunnels.
 
@@ -233,49 +215,50 @@ def get_tunnels(ngrok_path=None, timeout=4):
 
     If `ngrok` is not running, calling this method will start a process for the given path.
 
-    :param ngrok_path: A `ngrok` binary override (instead of using `pyngrok`'s).
-    :type ngrok_path: string, optional
-    :param timeout: The request timeout, in seconds.
-    :type timeout: float, optional
+    :param pyngrok_config: The Pyngrok configuration to use when with `ngrok`.
+    :type pyngrok_config: PyngrokConfig, optional
     :return: The currently active `ngrok` tunnels.
     :rtype: list[NgrokTunnel]
     """
-    ngrok_path = ngrok_path if ngrok_path else DEFAULT_NGROK_PATH
+    if pyngrok_config is None:
+        pyngrok_config = PyngrokConfig()
 
-    api_url = get_ngrok_process(ngrok_path).api_url
+    api_url = get_ngrok_process(pyngrok_config).api_url
 
     tunnels = []
-    for tunnel in api_request("{}/api/{}".format(api_url, "tunnels"), timeout=timeout)["tunnels"]:
+    for tunnel in api_request("{}/api/{}".format(api_url, "tunnels"), method="GET", data=None,
+                              timeout=pyngrok_config.request_timeout)["tunnels"]:
         tunnels.append(NgrokTunnel(tunnel))
 
     return tunnels
 
 
-def kill(ngrok_path=None):
+def kill(pyngrok_config=None):
     """
     Terminate the `ngrok` processes, if running, for the given path. This method will not block, it will just issue
     a kill request.
 
-    :param ngrok_path: A `ngrok` binary override (instead of using `pyngrok`'s).
-    :type ngrok_path: string, optional
+    :param pyngrok_config: The Pyngrok configuration to use when with `ngrok`.
+    :type pyngrok_config: PyngrokConfig, optional
     """
-    ngrok_path = ngrok_path if ngrok_path else DEFAULT_NGROK_PATH
+    if pyngrok_config is None:
+        pyngrok_config = PyngrokConfig()
 
-    process.kill_process(ngrok_path)
+    process.kill_process(pyngrok_config.ngrok_path)
 
 
 def api_request(url, method="GET", data=None, params=None, timeout=4):
     """
     Invoke an API request to the given URI, returning JSON data from the response as a dict.
 
-    :param url: The request URI.
-    :type url: string
-    :param method: The HTTP method, defaults to "GET".
-    :type method: string, optional
+    :param url: The request URL.
+    :type url: str
+    :param method: The HTTP method.
+    :type method: str, optional
     :param data: The request body.
     :type data: dict, optional
     :param params: The URL parameters.
-    :type params: list, optional
+    :type params: list[str], optional
     :param timeout: The request timeout, in seconds.
     :type timeout: float, optional
     :return: The response from the request.
@@ -284,7 +267,7 @@ def api_request(url, method="GET", data=None, params=None, timeout=4):
     if params is None:
         params = []
 
-    if not url.lower().startswith('http'):
+    if not url.lower().startswith("http"):
         raise PyngrokSecurityError("URL must start with 'http': {}".format(url))
 
     data = json.dumps(data).encode("utf-8") if data else None
@@ -333,14 +316,14 @@ def run(args=None):
     Start a blocking `ngrok` process with the default binary and the system's command line args.
 
     :param args: Arguments to be passed to the `ngrok` process.
-    :type args: list, optional
+    :type args: list[str], optional
     """
     if args is None:
         args = []
 
-    ensure_ngrok_installed(DEFAULT_NGROK_PATH)
+    ensure_ngrok_installed(conf.DEFAULT_NGROK_PATH)
 
-    process.run_process(DEFAULT_NGROK_PATH, args)
+    process.run_process(conf.DEFAULT_NGROK_PATH, args)
 
 
 def main():
@@ -355,5 +338,5 @@ def main():
         print("pyngrok version {}".format(__version__))
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
